==> eyflutter_uikit-0.0.8%2B8.tar.gz: lib/uikit/state/base_form_state.dart <==
import 'package:eyflutter_core/kit/click_event.dart';
import 'package:eyflutter_core/kit/parts/network_manager.dart';
import 'package:eyflutter_core/kit/utils/plugin_utils.dart';
import 'package:eyflutter_core/kit/utils/string/string_extension.dart';
import 'package:eyflutter_core/provider/models/provider_model.dart';
import 'package:eyflutter_uikit/uikit/bars/action_bar.dart';
import 'package:eyflutter_uikit/uikit/images/asset_image_provider.dart';
import 'package:eyflutter_uikit/uikit/refresh/cus_refresh_controller.dart';
import 'package:eyflutter_uikit/uikit/refresh/refresh_widget.dart';
import 'package:eyflutter_uikit/uikit/state/base_state.dart';
import 'package:eyflutter_uikit/uikit/state/state_utils.dart';
import 'package:eyflutter_uikit/uikit/widgets/scroll_vertical_view.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';

/// 消费组件构建器
typedef Widget ConsumerWidgetBuilder<M extends ChangeNotifier>(BuildContext context, M model);

/// 页面固件返回事件
typedef OnBackPressed = Future<bool> Function();

class _StateWidget {
  _StateWidget._();

  static Widget subscribeNotifierWidget<M extends ProviderModel>(M model, ConsumerWidgetBuilder<M> childBuilder) {
    return StateUtils.instance.subscribeNotifierWidget<M>(model, childBuilder);
  }
}

/// 组件构建器
abstract class ConsumerBuilder<M extends ProviderModel, P extends ProviderModel> with UniversalModel<P> {
  late M _model;

  M get model => _model;

  void setParentModel(P parent) {
    setParent(parent);
  }

  Widget builder(BuildContext context);

  /// 订阅消费组件
  /// [model] 订阅模型
  /// [childBuilder] 子组件构建器
  Widget subscribeNotifierWidget(M model, ConsumerWidgetBuilder<M> childBuilder) {
    _model = model;
    parent?.addModel<M>(model.modKey ?? "", model);
    model.setParent(super.parent);
    return _StateWidget.subscribeNotifierWidget<M>(model, childBuilder);
  }
}

abstract class BaseFormModel extends ProviderModel {
  //刷新组件controller
  CusRefreshController? refreshController;
  late Map<String, dynamic> _arguments;
  late BuildContext context;

  void setArguments(Map<String, dynamic> arguments) {
    this._arguments = arguments;
  }

  Map<String, dynamic> get arguments => _arguments;

  void onPreCreate(BuildContext context, Map<String, dynamic> arguments) {}

  void onInitData(BuildContext context, Map<String, dynamic> arguments);

  /// 刷新数据
  void refresh(BuildContext context) {
    //网络异常时显示失败视图并结束请求
    NetworkManager.instance.isConnected().then((netStatus) {
      if (netStatus ?? true) {
        refreshController?.isAutoTriggerLoad = true;
        onInitData(context, _arguments);
      } else {
        refreshController?.finishRefresh(success: true, noMore: false);
      }
    });
  }
}

abstract class BaseFormState<T extends StatefulWidget, P extends BaseFormModel> extends BaseState<T> {
  late P _model;
  ScrollController _scrollController = ScrollController();

  /// 创建notify model
  P get createModel;

  /// 获取模型对象
  P get model => _model;

  /// 页面背景颜色
  Color? get backgroundColor => Color(0xFFF4F4F4);

  /// 背景图片地址
  String get backgroundImage => "";

  /// 背景图片对齐方式
  AlignmentGeometry get backgroundAlignment => Alignment.bottomCenter;

  /// 背景宽度
  double? backgroundWidth;

  /// 背景高度
  double? backgroundHeight;

  /// 是否启用刷新
  bool get enableRefresh => false;

  /// 是否启用滑动监听
  bool get enableSlideListen => false;

  @override
  void onPreCreate(BuildContext context, String routeName, Map<String, dynamic> arguments) {
    _model = createModel;
    _model.setArguments(arguments);
    _model.onPreCreate(context, arguments);
  }

  @override
  void onCreate(BuildContext context, String routeName, Map<String, dynamic> arguments) {
    _model.context = context;
    _model.onInitData(context, arguments);
  }

  ActionBar? onActionBar() {
    return null;
  }

  OnBackPressed? onBackClick() {
    return null;
  }

  /// 构建页面顶部视图
  void onBuildHeadView(List<Widget> lst) {}

  /// 构建渲染对象
  Widget onRenderObject() {
    return Container();
  }

  /// 定位视图
  void onPositionedObjects(List<Widget> list) {}

  /// 构建底部视图
  Widget onBuildBottomView(BuildContext context) {
    return Container();
  }

  /// 左滑
  void onLeftSlide(BuildContext context) {}

  /// 上滑
  void onUpSlide(BuildContext context) {}

  /// 下滑
  void onDownSlide(BuildContext context) {}

  List<Widget> _stackList() {
    List<Widget> lst = [];
    if (this.backgroundImage.isNotEmptyString) {
      lst.add(Container(
        width: this.backgroundWidth,
        height: this.backgroundHeight,
        decoration: BoxDecoration(
          image: DecorationImage(
            image: AssetImageProvider.render(this.backgroundImage, isProjectImage: !PluginUtils.instance.isRunPlugin),
            alignment: this.backgroundAlignment,
            fit: BoxFit.scaleDown,
          ),
        ),
      ));
    }
    lst.add(onBodyObject());
    onPositionedObjects(lst);
    return lst;
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    return StateUtils.instance.subscribeNotifierWidget(model, (context, model) {
      return super.overrideLocalizations(
        context: context,
        child: WillPopScope(
          child: Scaffold(
            appBar: onActionBar(),
            backgroundColor: this.backgroundColor,
            body: Stack(
              children: _stackList(),
            ),
          ),
          onWillPop: () {
            var backClick = onBackClick();
            if (backClick != null) {
              return backClick();
            }
            return Future.value(true);
          },
        ),
      );
    });
  }

  /// 重写窗体对象不包括top bottom视图
  Widget onBodyObject() {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: _getBodyElements(),
    );
  }

  List<Widget> _getBodyElements() {
    List<Widget> lst = [];
    onBuildHeadView(lst);
    lst.add(enableSlideListen ? _addViewListen(context) : _onBodyObject(true));
    lst.add(onBuildBottomView(context));
    return lst;
  }

  double _lastY = 0.0;
  double _lastX = 0.0;

  Widget _addViewListen(BuildContext context) {
    return Expanded(
      child: Listener(
        onPointerDown: (PointerDownEvent event) {
          _lastX = event.position.dx;
          _lastY = event.position.dy;
        },
        onPointerMove: (PointerMoveEvent event) {
          var distanceX = event.position.dx;
          var distanceY = event.position.dy;
          var diffX = distanceX - _lastX;
          var diffY = distanceY - _lastY;
          var diffXAbs = diffX.abs();
          if (diffXAbs > 30 && diffX < 0) {
            if (!ClickEvent.isFastClick()) {
              onLeftSlide(context);
            }
          } else {
            var diffYAbs = diffY.abs();
            if (diffYAbs > 30 && diffY > 0) {
              if (!ClickEvent.isFastClick()) {
                onDownSlide(context);
              }
            } else if (diffYAbs > 30 && diffY < 0) {
              if (!ClickEvent.isFastClick()) {
                onUpSlide(context);
              }
            }
          }
        },
        child: _onBodyObject(false),
      ),
    );
  }

  Widget _buildScrollView(bool isExpand) {
    if (isExpand) {
      return Expanded(
        child: ScrollVerticalView(
          controller: _scrollController,
          child: onRenderObject(),
        ),
      );
    } else {
      return ScrollVerticalView(
        controller: _scrollController,
        child: onRenderObject(),
      );
    }
  }

  Widget _onBodyObject(bool isExpand) {
    if (this.enableRefresh) {
      return _buildRefreshWidget(context);
    } else {
      return _buildScrollView(isExpand);
    }
  }

  Widget _buildRefreshWidget(BuildContext context) {
    return RefreshWidget(
        listStyle: RefreshListStyle.custom,
        scrollMaxRatioLoad: 1,
        child: onRenderObject(),
        enableRefresh: this.enableRefresh,
        enableLoad: false,
        initCall: (controller) {
          model.refreshController = controller;
        },
        onRefresh: (EasyRefreshController controller) async {
          model.refresh(context);
        });
  }
}

==> eyflutter_uikit-0.1.4%2B8.tar.gz: lib/uikit/state/base_form_state.dart <==
import 'package:eyflutter_core/kit/click_event.dart';
import 'package:eyflutter_core/kit/parts/network_manager.dart';
import 'package:eyflutter_core/kit/utils/plugin_utils.dart';
import 'package:eyflutter_core/kit/utils/string/string_extension.dart';
import 'package:eyflutter_core/provider/models/provider_model.dart';
import 'package:eyflutter_uikit/uikit/images/asset_image_provider.dart';
import 'package:eyflutter_uikit/uikit/refresh/cus_refresh_controller.dart';
import 'package:eyflutter_uikit/uikit/refresh/refresh_widget.dart';
import 'package:eyflutter_uikit/uikit/state/base_state.dart';
import 'package:eyflutter_uikit/uikit/state/state_utils.dart';
import 'package:eyflutter_uikit/uikit/widgets/scroll_vertical_view.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';

/// 消费组件构建器
typedef Widget ConsumerWidgetBuilder<M extends ChangeNotifier>(BuildContext context, M model);

/// 页面固件返回事件
typedef OnBackPressed = Future<bool> Function();

class _StateWidget {
  _StateWidget._();

  static Widget subscribeNotifierWidget<M extends ProviderModel>(M model, ConsumerWidgetBuilder<M> childBuilder) {
    return StateUtils.instance.subscribeNotifierWidget<M>(model, childBuilder);
  }
}

/// 组件构建器
abstract class ConsumerBuilder<M extends ProviderModel, P extends ProviderModel> with UniversalModel<P> {
  late M _model;

  M get model => _model;

  void setParentModel(P parent) {
    setParent(parent);
  }

  Widget builder(BuildContext context);

  /// 订阅消费组件
  /// [model] 订阅模型
  /// [childBuilder] 子组件构建器
  Widget subscribeNotifierWidget(M model, ConsumerWidgetBuilder<M> childBuilder) {
    _model = model;
    parent?.addModel<M>(model.modKey ?? "", model);
    model.setParent(super.parent);
    return _StateWidget.subscribeNotifierWidget<M>(model, childBuilder);
  }
}

abstract class BaseFormModel extends ProviderModel {
  //刷新组件controller
  CusRefreshController? refreshController;
  late Map<String, dynamic> _arguments;
  late BuildContext context;

  void setArguments(Map<String, dynamic> arguments) {
    this._arguments = arguments;
  }

  Map<String, dynamic> get arguments => _arguments;

  void onPreCreate(BuildContext context, Map<String, dynamic> arguments) {}

  void onInitData(BuildContext context, Map<String, dynamic> arguments);

  /// 刷新数据
  void refresh(BuildContext context) {
    //网络异常时显示失败视图并结束请求
    NetworkManager.instance.isConnected().then((netStatus) {
      if (netStatus ?? true) {
        refreshController?.isAutoTriggerLoad = true;
        onInitData(context, _arguments);
      } else {
        refreshController?.finishRefresh(success: true, noMore: false);
      }
    });
  }
}

abstract class BaseFormState<T extends StatefulWidget, P extends BaseFormModel> extends BaseState<T> {
  late P _model;
  ScrollController scrollController = ScrollController();

  /// 创建notify model
  P get createModel;

  /// 获取模型对象
  P get model => _model;

  /// 背景图片地址
  String get backgroundImage => "";

  /// 背景图片填充方式
  BoxFit get backgroundImageFix => BoxFit.scaleDown;

  /// 背景图片平铺方式
  ImageRepeat get backgroundRepeat => ImageRepeat.noRepeat;

  /// 背景图片对齐方式
  AlignmentGeometry get backgroundAlignment => Alignment.bottomCenter;

  /// 背景宽度
  double? backgroundWidth;

  /// 背景高度
  double? backgroundHeight;

  /// 背景组件与backgroundColor和backgroundImage互斥
  Widget? backgroundWidget;

  /// 是否启用刷新
  bool get enableRefresh => false;

  /// 是否启用滑动监听
  bool get enableSlideListen => false;

  @override
  void onPreCreate(BuildContext context, String routeName, Map<String, dynamic> arguments) {
    _model = createModel;
    _model.setArguments(arguments);
    _model.onPreCreate(context, arguments);
  }

  @override
  void onCreate(BuildContext context, String routeName, Map<String, dynamic> arguments) {
    _model.context = context;
    _model.onInitData(context, arguments);
  }

  /// 构建页面顶部视图
  void onBuildHeadView(List<Widget> lst) {}

  /// 构建渲染对象
  Widget onRenderObject() {
    return Container();
  }

  /// 定位视图
  void onPositionedObjects(List<Widget> list) {}

  /// 构建底部视图
  Widget onBuildBottomView(BuildContext context) {
    return Container();
  }

  /// 左滑
  void onLeftSlide(BuildContext context) {}

  /// 上滑
  void onUpSlide(BuildContext context) {}

  /// 下滑
  void onDownSlide(BuildContext context) {}

  List<Widget> _stackList() {
    List<Widget> lst = [];
    if (this.backgroundWidget == null) {
      if (this.backgroundImage.isNotEmptyString) {
        lst.add(Container(
          width: this.backgroundWidth,
          height: this.backgroundHeight,
          decoration: BoxDecoration(
            image: DecorationImage(
              image: AssetImageProvider.render(this.backgroundImage, isProjectImage: !PluginUtils.instance.isRunPlugin),
              alignment: this.backgroundAlignment,
              fit: this.backgroundImageFix,
              repeat: this.backgroundRepeat,
            ),
          ),
        ));
      }
    } else {
      lst.add(backgroundWidget!);
    }
    lst.add(onBodyObject());
    onPositionedObjects(lst);
    return lst;
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    return StateUtils.instance.subscribeNotifierWidget(model, (context, model) {
      return super.overrideLocalizations(
        context: context,
        child: Stack(
          children: _stackList(),
        ),
      );
    });
  }

  /// 重写窗体对象不包括top bottom视图
  Widget onBodyObject() {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: _getBodyElements(),
    );
  }

  List<Widget> _getBodyElements() {
    List<Widget> lst = [];
    onBuildHeadView(lst);
    lst.add(enableSlideListen ? _addViewListen(context) : _onBodyObject(true));
    lst.add(onBuildBottomView(context));
    return lst;
  }

  double _lastY = 0.0;
  double _lastX = 0.0;

  Widget _addViewListen(BuildContext context) {
    return Expanded(
      child: Listener(
        onPointerDown: (PointerDownEvent event) {
          _lastX = event.position.dx;
          _lastY = event.position.dy;
        },
        onPointerMove: (PointerMoveEvent event) {
          var distanceX = event.position.dx;
          var distanceY = event.position.dy;
          var diffX = distanceX - _lastX;
          var diffY = distanceY - _lastY;
          var diffXAbs = diffX.abs();
          if (diffXAbs > 30 && diffX < 0) {
            if (!ClickEvent.isFastClick()) {
              onLeftSlide(context);
            }
          } else {
            var diffYAbs = diffY.abs();
            if (diffYAbs > 30 && diffY > 0) {
              if (!ClickEvent.isFastClick()) {
                onDownSlide(context);
              }
            } else if (diffYAbs > 30 && diffY < 0) {
              if (!ClickEvent.isFastClick()) {
                onUpSlide(context);
              }
            }
          }
        },
        child: _onBodyObject(false),
      ),
    );
  }

  Widget _buildScrollView(bool isExpand) {
    if (isExpand) {
      return Expanded(
        child: ScrollVerticalView(
          controller: scrollController,
          child: onRenderObject(),
        ),
      );
    } else {
      return ScrollVerticalView(
        controller: scrollController,
        child: onRenderObject(),
      );
    }
  }

  Widget _onBodyObject(bool isExpand) {
    if (this.enableRefresh) {
      return _buildRefreshWidget(context);
    } else {
      return _buildScrollView(isExpand);
    }
  }

  Widget _buildRefreshWidget(BuildContext context) {
    return RefreshWidget(
        listStyle: RefreshListStyle.custom,
        scrollMaxRatioLoad: 1,
        child: onRenderObject(),
        enableRefresh: this.enableRefresh,
        enableLoad: false,
        initCall: (controller) {
          model.refreshController = controller;
        },
        onRefresh: (EasyRefreshController controller) async {
          model.refresh(context);
        });
  }
}
